import os
import threading
from contextlib import contextmanager
from http.server import BaseHTTPRequestHandler, HTTPServer
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Generator, Optional, Type
from unittest import mock
from urllib.parse import urlparse

from kagglehub.clients import (
    KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME,
    KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME,
    KAGGLE_JWT_TOKEN_ENV_VAR_NAME,
)
from kagglehub.colab_cache_resolver import COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME
from kagglehub.config import CACHE_FOLDER_ENV_VAR_NAME, KAGGLE_API_ENDPOINT_ENV_VAR_NAME, TBE_RUNTIME_ADDR_ENV_VAR_NAME
from kagglehub.env import KAGGLE_NOTEBOOK_ENV_VAR_NAME
from kagglehub.handle import ResourceHandle
from kagglehub.kaggle_cache_resolver import KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME


def get_test_file_path(relative_path: str) -> str:
    return os.path.join(Path(__file__).parent, "data", relative_path)


@contextmanager
def create_test_cache() -> Generator[str, None, None]:
    with TemporaryDirectory() as d:
        with mock.patch.dict(os.environ, {CACHE_FOLDER_ENV_VAR_NAME: d}):
            yield d


@contextmanager
def create_test_http_server(
    handler_class: Type[BaseHTTPRequestHandler], endpoint: Optional[str] = None
) -> Generator[HTTPServer, None, None]:
    if endpoint is None:
        endpoint = os.getenv(KAGGLE_API_ENDPOINT_ENV_VAR_NAME)

    test_server_address = urlparse(endpoint)
    if not test_server_address.hostname or not test_server_address.port:
        msg = f"Invalid test server address: {endpoint}. You must specify a hostname & port"
        raise ValueError(msg)
    with HTTPServer((test_server_address.hostname, test_server_address.port), handler_class) as httpd:
        threading.Thread(target=httpd.serve_forever).start()

        try:
            yield httpd
        finally:
            httpd.shutdown()


@contextmanager
def create_test_jwt_http_server(handler_class: Type[BaseHTTPRequestHandler]) -> Generator[HTTPServer, None, None]:
    with TemporaryDirectory() as cache_mount_folder:
        with mock.patch.dict(
            os.environ,
            {
                KAGGLE_NOTEBOOK_ENV_VAR_NAME: "Interactive",
                KAGGLE_JWT_TOKEN_ENV_VAR_NAME: "foo jwt token",
                KAGGLE_DATA_PROXY_TOKEN_ENV_VAR_NAME: "foo proxy token",
                KAGGLE_CACHE_MOUNT_FOLDER_ENV_VAR_NAME: cache_mount_folder,
            },
        ):
            endpoint = os.getenv(KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME)
            test_server_address = urlparse(endpoint)
            if not test_server_address.hostname or not test_server_address.port:
                msg = f"Invalid JWT test server address: {endpoint}. You must specify a hostname & port"
                raise ValueError(msg)
            with HTTPServer((test_server_address.hostname, test_server_address.port), handler_class) as httpd:
                threading.Thread(target=httpd.serve_forever).start()

                try:
                    yield httpd
                finally:
                    httpd.shutdown()


@contextmanager
def create_test_server_colab(handler_class: Type[BaseHTTPRequestHandler]) -> Generator[HTTPServer, None, None]:
    with TemporaryDirectory() as cache_mount_folder:
        with mock.patch.dict(
            os.environ,
            {
                TBE_RUNTIME_ADDR_ENV_VAR_NAME: "localhost:7779",
                COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME: cache_mount_folder,
            },
        ):
            endpoint = os.getenv(TBE_RUNTIME_ADDR_ENV_VAR_NAME)
            test_server_address = urlparse(f"http://{endpoint}")
            if not test_server_address.hostname or not test_server_address.port:
                msg = f"Invalid test server address: {endpoint}. You must specify a hostname & port"
                raise ValueError(msg)
            with HTTPServer((test_server_address.hostname, test_server_address.port), handler_class) as httpd:
                threading.Thread(target=httpd.serve_forever).start()
                try:
                    yield httpd
                finally:
                    httpd.shutdown()


class InvalidResourceHandle(ResourceHandle):
    def __init__(self):
        self.owner = "invalid"

    def to_url(self) -> str:
        return "invalid"



import os

# Ensures tests do not try to read your own kaggle credentials.
os.environ["KAGGLE_CONFIG_DIR"] = "/some-missing-directory"

# All APIs call in tests should go to a local test server.
os.environ["KAGGLE_API_ENDPOINT"] = "http://localhost:7777"

# All JWT handlers calls in tests should go to a local test server.
os.environ["KAGGLE_DATA_PROXY_URL"] = "http://localhost:7778"



import json
import os
from http.server import BaseHTTPRequestHandler
from pathlib import Path
from unittest import mock

import requests

import kagglehub
from kagglehub.clients import ColabClient
from kagglehub.colab_cache_resolver import COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME
from kagglehub.config import DISABLE_COLAB_CACHE_ENV_VAR_NAME
from tests.fixtures import BaseTestCase

from .utils import create_test_server_colab

INVALID_ARCHIVE_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/bad-archive-variation/1"
VERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b/1"
LATEST_MODEL_VERSION = 2
UNVERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b"
TEST_FILEPATH = "config.json"
UNAVAILABLE_MODEL_HANDLE = "unavailable/model/handle/colab/1"


class ColabTBERuntimeHandler(BaseHTTPRequestHandler):
    def do_HEAD(self) -> None:  # noqa: N802
        self.send_response(200)

    def do_POST(self) -> None:  # noqa: N802
        content_length = int(self.headers["Content-Length"])
        request = json.loads(self.rfile.read(content_length))
        version = LATEST_MODEL_VERSION
        if "version" in request:
            version = request["version"]

        slug = f"{request['model']}/{request['framework']}/{request['variation']}/{version}"

        if self.path.endswith(ColabClient.IS_SUPPORTED_PATH):
            if request["owner"] == "unavailable":
                self.send_response(404)
                self.send_header("Content-type", "application/json")
                self.end_headers()
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()

        elif self.path.endswith(ColabClient.MOUNT_PATH):
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()

            # Load the files
            cache_mount_folder = os.getenv(COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME)
            base_path = f"{cache_mount_folder}/{slug}"
            os.makedirs(base_path, exist_ok=True)

            Path(f"{base_path}/config.json").touch()

            if version == LATEST_MODEL_VERSION:
                # The latest version has an extra file.
                Path(f"{base_path}/model.keras").touch()

            self.wfile.write(
                bytes(
                    json.dumps(
                        {
                            "slug": slug,
                        }
                    ),
                    "utf-8",
                )
            )
        else:
            self.send_response(404)
            self.wfile.write(bytes(f"Unhandled path: {self.path}", "utf-8"))


class TestColabCacheModelDownload(BaseTestCase):
    def test_unversioned_model_download(self) -> None:
        with create_test_server_colab(ColabTBERuntimeHandler):
            model_path = kagglehub.model_download(UNVERSIONED_MODEL_HANDLE)
            self.assertTrue(model_path.endswith("/2"))
            self.assertEqual(["config.json", "model.keras"], sorted(os.listdir(model_path)))

    def test_versioned_model_download(self) -> None:
        with create_test_server_colab(ColabTBERuntimeHandler):
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE)
            self.assertTrue(model_path.endswith("/1"))
            self.assertEqual(["config.json"], sorted(os.listdir(model_path)))

    def test_versioned_model_download_with_path(self) -> None:
        with create_test_server_colab(ColabTBERuntimeHandler):
            model_file_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE, "config.json")
            self.assertTrue(model_file_path.endswith("config.json"))
            self.assertTrue(os.path.isfile(model_file_path))

    def test_unversioned_model_download_with_path(self) -> None:
        with create_test_server_colab(ColabTBERuntimeHandler):
            model_file_path = kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, "config.json")
            self.assertTrue(model_file_path.endswith("config.json"))
            self.assertTrue(os.path.isfile(model_file_path))

    def test_versioned_model_download_with_missing_file_raises(self) -> None:
        with create_test_server_colab(ColabTBERuntimeHandler):
            with self.assertRaises(ValueError):
                kagglehub.model_download(VERSIONED_MODEL_HANDLE, "missing.txt")

    def test_unversioned_model_download_with_missing_file_raises(self) -> None:
        with create_test_server_colab(ColabTBERuntimeHandler):
            with self.assertRaises(ValueError):
                kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, "missing.txt")

    def test_colab_resolver_skipped_when_disable_colab_cache_env_var_name(self) -> None:
        with mock.patch.dict(os.environ, {DISABLE_COLAB_CACHE_ENV_VAR_NAME: "true"}):
            with create_test_server_colab(ColabTBERuntimeHandler):
                # Assert that a ConnectionError is set (uses HTTP server which is not set)
                with self.assertRaises(requests.exceptions.ConnectionError):
                    kagglehub.model_download(VERSIONED_MODEL_HANDLE)

    def test_colab_resolver_skipped_when_model_not_present(self) -> None:
        with create_test_server_colab(ColabTBERuntimeHandler):
            # Assert that a ConnectionError is set (uses HTTP server which is not set)
            with self.assertRaises(requests.exceptions.ConnectionError):
                kagglehub.model_download(UNAVAILABLE_MODEL_HANDLE)

    def test_versioned_model_download_bad_handle_raises(self) -> None:
        with self.assertRaises(ValueError):
            kagglehub.model_download("bad handle")



import json
import os
from http.server import BaseHTTPRequestHandler
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import ClassVar, List

from kagglehub.exceptions import BackendError
from kagglehub.gcs_upload import MAX_FILES_TO_UPLOAD, TEMP_ARCHIVE_FILE
from kagglehub.models import model_upload
from tests.fixtures import BaseTestCase

from .utils import create_test_http_server

GET_INSTANCE = "/models/metaresearch/llama-2/pyTorch/1/get"
GET_MODEL = "/models/metaresearch/llama-2/get"
CREATE_MODEL = "/models/create/new"
MODEL_HANDLE = "metaresearch/llama-2/pyTorch/1"
TEMP_TEST_FILE = "temp_test_file"
APACHE_LICENSE = "Apache 2.0"
ALLOWED_LICENSE_VALUES = (APACHE_LICENSE, None)


class KaggleAPIHandler(BaseHTTPRequestHandler):
    UPLOAD_BLOB_FILE_NAMES: ClassVar[List[str]] = []

    def do_HEAD(self) -> None:  # noqa: N802
        self.send_response(200)

    def do_GET(self) -> None:  # noqa: N802
        if self.path == f"/api/v1{GET_MODEL}":
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(bytes(json.dumps({"message": "Model exists!"}), "utf-8"))
        elif self.path == f"/api/v1{GET_INSTANCE}":
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(bytes(json.dumps({"message": "Instance exists!"}), "utf-8"))
        else:
            self.send_response(404)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            response_data = {"message": "Some response data"}
            self.wfile.write(bytes(json.dumps(response_data), "utf-8"))

    def do_POST(self) -> None:  # noqa: N802
        instance_or_version = self.path.split("/")[-1]
        content_length = int(self.headers["Content-Length"])
        post_data = self.rfile.read(content_length)
        data = json.loads(post_data.decode("utf-8"))

        # Extracting the 'name' from the data
        name = data.get("name", None)
        if self.path == f"/api/v1{CREATE_MODEL}":
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(json.dumps({"status": "success", "message": "Model created successfully"}).encode("utf-8"))
        elif instance_or_version in ("instance", "version"):
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            if data.get("licenseName") not in ALLOWED_LICENSE_VALUES:
                error_message = json.dumps({"error": f"bad: {self.path}"})
                self.wfile.write(bytes(error_message, "utf-8"))
            else:
                response = {"status": "success", "message": "Model Instance/Version created successfully"}
                self.wfile.write(json.dumps(response).encode("utf-8"))
        elif self.path == "/api/v1/blobs/upload":
            KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES.append(name)
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(
                json.dumps(
                    {
                        "token": "dummy",
                        "createUrl": "http://localhost:7778",
                        "status": "success",
                        "message": "Here is your token and Url",
                    }
                ).encode("utf-8")
            )
        else:
            self.send_response(200)
            self.send_header("Content-Type", "application/json")
            self.end_headers()
            error_message = json.dumps({"error": f"bad: {self.path}"})
            self.wfile.write(bytes(error_message, "utf-8"))


class GcsAPIHandler(BaseHTTPRequestHandler):
    simulate_308 = False
    put_requests_count = 0

    def do_PUT(self) -> None:  # noqa: N802
        GcsAPIHandler.put_requests_count += 1
        if GcsAPIHandler.simulate_308:
            # Simulate "308 Resume Incomplete" response
            self.send_response(308)
            self.send_header("Content-type", "application/json")
            self.send_header("Range", "bytes=0-499")
            self.end_headers()
            GcsAPIHandler.simulate_308 = False
        else:
            # Simulate successful upload
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(json.dumps({"status": "success", "message": "File uploaded"}).encode("utf-8"))


class TestModelUpload(BaseTestCase):
    def setUp(self) -> None:
        # Resetting shared variables in GcsAPIHandler
        GcsAPIHandler.simulate_308 = False
        GcsAPIHandler.put_requests_count = 0

        # Resetting any other necessary setup for KaggleAPIHandler
        KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES = []

    def test_model_upload_with_invalid_handle(self) -> None:
        with create_test_http_server(KaggleAPIHandler):
            with self.assertRaises(ValueError):
                with TemporaryDirectory() as temp_dir:
                    test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                    test_filepath.touch()  # Create a temporary file in the temporary directory
                    model_upload("invalid/invalid/invalid", temp_dir, APACHE_LICENSE, "model_type")
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_TEST_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_instance_with_valid_handle(self) -> None:
        # execution path: get_model -> create_model -> get_instance -> create_version
        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                    test_filepath.touch()  # Create a temporary file in the temporary directory
                    model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE, "model_type")
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_ARCHIVE_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_instance_with_nested_directories(self) -> None:
        # execution path: get_model -> create_model -> get_instance -> create_version
        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    # Create a nested directory structure
                    nested_dir = Path(temp_dir) / "nested"
                    nested_dir.mkdir()

                    # Create a temporary file in the nested directory
                    test_filepath = nested_dir / TEMP_TEST_FILE
                    test_filepath.touch()
                    model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE, "model_type")
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_ARCHIVE_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_version_with_valid_handle(self) -> None:
        # execution path: get_model -> get_instance -> create_instance

        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                    test_filepath.touch()  # Create a temporary file in the temporary directory
                    model_upload("metaresearch/llama-2/pyTorch/7b", temp_dir, APACHE_LICENSE, "model_type")
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_ARCHIVE_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_with_too_many_files(self) -> None:
        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    # Create more than 50 temporary files in the directory
                    for i in range(MAX_FILES_TO_UPLOAD + 1):
                        test_filepath = Path(temp_dir) / f"temp_test_file_{i}"
                        test_filepath.touch()

                    model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE, "model_type")
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_ARCHIVE_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_resumable(self) -> None:
        GcsAPIHandler.simulate_308 = True  # Enable simulation of 308 response for this test

        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                    test_filepath.touch()
                    with open(test_filepath, "wb") as f:
                        f.write(os.urandom(1000))

                    model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, APACHE_LICENSE, "model_type")

                    # Check that GcsAPIHandler received two PUT requests
                    self.assertEqual(GcsAPIHandler.put_requests_count, 2)
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_ARCHIVE_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_with_none_license(self) -> None:
        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                    test_filepath.touch()  # Create a temporary file in the temporary directory
                    model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, None, "model_type")
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_ARCHIVE_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_without_license(self) -> None:
        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                    test_filepath.touch()  # Create a temporary file in the temporary directory
                    model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, version_notes="model_type")
                    self.assertEqual(len(KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES), 1)
                    self.assertIn(TEMP_ARCHIVE_FILE, KaggleAPIHandler.UPLOAD_BLOB_FILE_NAMES)

    def test_model_upload_with_invalid_license_fails(self) -> None:
        with create_test_http_server(KaggleAPIHandler):
            with create_test_http_server(GcsAPIHandler, "http://localhost:7778"):
                with TemporaryDirectory() as temp_dir:
                    test_filepath = Path(temp_dir) / TEMP_TEST_FILE
                    test_filepath.touch()  # Create a temporary file in the temporary directory
                    with self.assertRaises(BackendError):
                        model_upload("metaresearch/new-model/pyTorch/new-variation", temp_dir, "Invalid License")



import unittest
from unittest.mock import MagicMock, patch

from integration_tests.test_model_upload import retry


class FunctionToTest:
    def __init__(self):
        self.attempt = 0

    def __call__(self, success_on_attempt: int) -> str:
        """A simple function that raises an exception until it reaches the successful attempt."""
        if self.attempt < success_on_attempt:
            self.attempt += 1
            value_error_message = "Test error"
            raise ValueError(value_error_message)
        return "Success"


class TestRetryDecorator(unittest.TestCase):
    def setUp(self) -> None:
        self.function_to_test = FunctionToTest()

    @patch("integration_tests.test_model_upload.time.sleep", autospec=True)
    @patch("integration_tests.test_model_upload.logger.info", autospec=True)
    def test_retry_success_before_limit(self, mock_logger_info: MagicMock, mock_sleep: MagicMock) -> None:
        decorated = retry(times=3, delay_seconds=1)(self.function_to_test)
        result = decorated(2)
        self.assertEqual(result, "Success")
        self.assertEqual(self.function_to_test.attempt, 2)
        self.assertEqual(mock_sleep.call_count, 2)
        self.assertEqual(mock_logger_info.call_count, 2)

    @patch("integration_tests.test_model_upload.time.sleep", autospec=True)
    @patch("integration_tests.test_model_upload.logger.info", autospec=True)
    def test_retry_reaches_limit_raises_timeout(self, mock_logger_info: MagicMock, mock_sleep: MagicMock) -> None:
        decorated = retry(times=3, delay_seconds=2)(self.function_to_test)
        with self.assertRaises(TimeoutError):
            decorated(4)
        self.assertEqual(self.function_to_test.attempt, 3)
        self.assertEqual(mock_sleep.call_count, 2)
        self.assertEqual(mock_logger_info.call_count, 2)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from typing import List

from requests import HTTPError

from kagglehub import model_download

HANDLE = "keras/bert/keras/bert_tiny_en_uncased/2"


class TestModelDownload(unittest.TestCase):
    def list_files_recursively(self, path: str) -> List[str]:
        """List all files recursively in the given path.
        If the path is a file, return a list containing only that file.
        If the path is a directory, list all files recursively in that directory."""
        if os.path.isfile(path):
            directory = os.path.dirname(path)
            rel_file = os.path.relpath(path, directory)
            return [rel_file]

        files = []
        for root, _, filenames in os.walk(path):
            for filename in filenames:
                # Create a relative path from the directory and add it to the list
                rel_dir = os.path.relpath(root, path)
                rel_file = os.path.join(rel_dir, filename)
                if not rel_file.startswith("./?"):
                    files.append(rel_file)
        return sorted(files)

    def assert_files(self, path: str, expected_files: List[str]) -> None:
        """Assert that all expected files exist and are non-empty."""
        files = self.list_files_recursively(path)
        expected_files_sorted = sorted(expected_files)
        self.assertEqual(files, expected_files_sorted, "Downloaded files did not match expected files")

        # Assert that each file's size is greater than zero
        for file in files:
            if os.path.isfile(path):
                file_path = os.path.join(os.path.dirname(path), file)
            else:
                file_path = os.path.join(path, file)

            self.assertGreater(os.path.getsize(file_path), 0, f"File {file} is empty")

    def test_model_versioned_succeeds(self) -> None:
        actual_path = model_download(HANDLE)

        expected_files = [
            "assets/tokenizer/vocabulary.txt",
            "./config.json",
            "./metadata.json",
            "./model.weights.h5",
            "./tokenizer.json",
        ]
        self.assert_files(actual_path, expected_files)

    def test_model_unversioned_succeeds(self) -> None:
        unversioned_handle = "keras/bert/keras/bert_tiny_en_uncased"
        actual_path = model_download(unversioned_handle)

        expected_files = [
            "assets/tokenizer/vocabulary.txt",
            "./config.json",
            "./metadata.json",
            "./model.weights.h5",
            "./tokenizer.json",
        ]
        self.assert_files(actual_path, expected_files)

    def test_download_private_model_succeeds(self) -> None:
        actual_path = model_download("integrationtester/test-private-model/pyTorch/b0")

        expected_files = [
            "./efficientnet-b0.pth",
        ]

        self.assert_files(actual_path, expected_files)

    def test_download_multiple_files(self) -> None:
        file_paths = ["tokenizer.json", "config.json"]
        for p in file_paths:
            actual_path = model_download(HANDLE, path=p)
            self.assert_files(actual_path, [p])

    def test_download_with_incorrect_file_path(self) -> None:
        incorrect_path = "nonexistent/file/path"
        with self.assertRaises(HTTPError):
            model_download(HANDLE, path=incorrect_path)



import base64
import http.server
import io
import json
import logging
from unittest import mock

import kagglehub
from kagglehub.auth import _capture_logger_output, logger
from kagglehub.config import get_kaggle_credentials
from tests.fixtures import BaseTestCase

from .utils import create_test_http_server

GOOD_CREDENTIALS_USERNAME = "lastplacelarry"
GOOD_CREDENTIALS_API_KEY = "some-key"


class KaggleAPIHandler(http.server.BaseHTTPRequestHandler):
    def do_HEAD(self) -> None:  # noqa: N802
        self.send_response(200)

    def do_GET(self) -> None:  # noqa: N802
        if self.path == "/api/v1/hello":
            # Get the basic auth credentials attached to the request
            credentials = self.headers.get("Authorization", "").split(" ")[1]
            username, key = base64.b64decode(credentials.encode("utf-8")).decode("utf-8").split(":")

            # Compare to the expected good credentials username/key
            if username == GOOD_CREDENTIALS_USERNAME and key == GOOD_CREDENTIALS_API_KEY:
                self.send_response(200)
                self.send_header("Content-type", "application/json")
                self.end_headers()
                self.wfile.write(bytes(json.dumps({"message": "Hello from test server!"}), "utf-8"))
            else:
                self.send_response(200)
                self.send_header("Content-type", "application/json")
                self.end_headers()
                self.wfile.write(bytes(json.dumps({"code": 401}), "utf-8"))
        else:
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(bytes(json.dumps({"code": 404}), "utf-8"))


class TestAuth(BaseTestCase):
    def test_login_updates_global_credentials(self) -> None:
        # Simulate user input for credentials

        with create_test_http_server(KaggleAPIHandler):
            with mock.patch("builtins.input") as mock_input:
                mock_input.side_effect = ["lastplacelarry", "some-key"]
                kagglehub.login()

            # Verify that the global variable contains the updated credentials
            credentials = get_kaggle_credentials()
            if credentials is None:
                self.fail("Credentials should not be None")
            self.assertEqual("lastplacelarry", credentials.username)
            self.assertEqual("some-key", credentials.key)

    def test_login_updates_global_credentials_no_validation(self) -> None:
        # Simulate user input for credentials
        with mock.patch("builtins.input") as mock_input:
            mock_input.side_effect = ["lastplacelarry", "some-key"]
            kagglehub.login(validate_credentials=False)

        # Verify that the global variable contains the updated credentials
        credentials = get_kaggle_credentials()
        if credentials is None:
            self.fail("Credentials should not be None")
        self.assertEqual("lastplacelarry", credentials.username)
        self.assertEqual("some-key", credentials.key)

    def test_set_kaggle_credentials_raises_error_with_empty_username(self) -> None:
        with self.assertRaises(ValueError):
            with mock.patch("builtins.input") as mock_input:
                mock_input.side_effect = ["", "some-key"]
                kagglehub.login()

    def test_set_kaggle_credentials_raises_error_with_empty_api_key(self) -> None:
        with self.assertRaises(ValueError):
            with mock.patch("builtins.input") as mock_input:
                mock_input.side_effect = ["lastplacelarry", ""]
                kagglehub.login()

    def test_set_kaggle_credentials_raises_error_with_empty_username_api_key(self) -> None:
        with self.assertRaises(ValueError):
            with mock.patch("builtins.input") as mock_input:
                mock_input.side_effect = ["", ""]
                kagglehub.login()

    def test_login_returns_403_for_bad_credentials(self) -> None:
        output_stream = io.StringIO()
        handler = logging.StreamHandler(output_stream)
        logger.addHandler(handler)
        with create_test_http_server(KaggleAPIHandler):
            with mock.patch("builtins.input") as mock_input:
                mock_input.side_effect = ["invalid", "invalid"]
                kagglehub.login()

            captured_output = output_stream.getvalue()
            self.assertEqual(
                captured_output,
                "Invalid Kaggle credentials. You can check your credentials on the [Kaggle settings page](https://www.kaggle.com/settings/account).\n",
            )

    def test_capture_logger_output(self) -> None:
        with _capture_logger_output() as output:
            logger.info("This is an info message")
            logger.error("This is an error message")

        self.assertEqual(output.getvalue(), "This is an info message\nThis is an error message\n")



from typing import Callable, List


class MultiImplRegistry:
    """Utility class to inject multiple implementations of class.

    Each implementation must implement __call__ and is_supported with the same set of arguments. The registered
    implementations "is_supported" methods are called in reverse order under which they are registered. The first
    to return true is then invoked via __call__ and the result returned.
    """

    def __init__(self, name: str) -> None:
        self._name = name
        self._impls: List[Callable] = []

    def add_implementation(self, impl: Callable) -> None:
        self._impls += [impl]

    def __call__(self, *args, **kwargs):  # noqa: ANN002, ANN003
        fails = []
        for impl in reversed(self._impls):
            if impl.is_supported(*args, **kwargs):
                return impl(*args, **kwargs)
            else:
                fails.append(type(impl).__name__)

        msg = f"Missing implementation that supports: {self._name}(*{args!r}, **{kwargs!r}). Tried {fails!r}"
        raise RuntimeError(msg)


model_resolver = MultiImplRegistry("ModelResolver")
# TODO(b/313706281): Add a dataset_resolver here.



import logging
import os
from typing import Optional

from kagglehub.clients import ColabClient
from kagglehub.config import is_colab_cache_disabled
from kagglehub.exceptions import BackendError, NotFoundError
from kagglehub.handle import ModelHandle
from kagglehub.resolver import Resolver

COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME = "COLAB_CACHE_MOUNT_FOLDER"
DEFAULT_COLAB_CACHE_MOUNT_FOLDER = "/kaggle/input"

logger = logging.getLogger(__name__)


class ModelColabCacheResolver(Resolver[ModelHandle]):
    def is_supported(self, handle: ModelHandle, *_, **__) -> bool:  # noqa: ANN002, ANN003
        if ColabClient.TBE_RUNTIME_ADDR_ENV_VAR_NAME not in os.environ or is_colab_cache_disabled():
            return False

        api_client = ColabClient()
        data = {
            "owner": handle.owner,
            "model": handle.model,
            "framework": handle.framework,
            "variation": handle.variation,
        }

        if handle.is_versioned():
            # Colab treats version as int in the request
            data["version"] = handle.version  # type: ignore

        try:
            api_client.post(data, ColabClient.IS_SUPPORTED_PATH, handle)
        except NotFoundError:
            return False
        return True

    def __call__(self, h: ModelHandle, path: Optional[str] = None, *, force_download: Optional[bool] = False) -> str:
        if force_download:
            logger.warning("Ignoring invalid input: force_download flag cannot be used in a Colab notebook")

        if path:
            logger.info(f"Attaching '{path}' from model '{h}' to your Colab notebook...")
        else:
            logger.info(f"Attaching model '{h}' to your Colab notebook...")

        api_client = ColabClient()
        data = {
            "owner": h.owner,
            "model": h.model,
            "framework": h.framework,
            "variation": h.variation,
        }
        if h.is_versioned():
            # Colab treats version as int in the request
            data["version"] = h.version  # type: ignore

        response = api_client.post(data, ColabClient.MOUNT_PATH, h)

        if response is not None:
            if "slug" not in response:
                msg = "'slug' field missing from response"
                raise BackendError(msg)

            base_mount_path = os.getenv(COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME, DEFAULT_COLAB_CACHE_MOUNT_FOLDER)
            cached_path = f"{base_mount_path}/{response['slug']}"

            if path:
                cached_filepath = f"{cached_path}/{path}"
                if not os.path.exists(cached_filepath):
                    msg = (
                        f"'{path}' is not present in the model files. "
                        f"You can access the other files of the attached model at '{cached_path}'"
                    )
                    raise ValueError(msg)
                return cached_filepath
            return cached_path
        else:
            no_response = "No response received or response was empty."
            raise ValueError(no_response)



import logging
import os
import tempfile
import time
import unittest
import uuid
from functools import wraps
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Callable, Type, TypeVar

from kagglehub import model_upload, models_helpers
from kagglehub.config import get_kaggle_credentials
from kagglehub.exceptions import BackendError

LICENSE_NAME = "MIT"

logger = logging.getLogger(__name__)


ReturnType = TypeVar("ReturnType")


def retry(
    times: int = 5, delay_seconds: int = 5, exception_to_check: Type[Exception] = Exception
) -> Callable[[Callable[..., ReturnType]], Callable[..., ReturnType]]:
    def decorator(func: Callable[..., ReturnType]) -> Callable[..., ReturnType]:
        @wraps(func)
        def wrapper(*args: object, **kwargs: object) -> ReturnType:
            attempts = 0
            while attempts < times:
                try:
                    return func(*args, **kwargs)
                except exception_to_check as e:
                    attempts += 1
                    if attempts == times:
                        time_out_message = "Maximum retries reached without success."
                        raise TimeoutError(time_out_message) from e
                    logger.info(f"Attempt {attempts} failed: {e}. Retrying in {delay_seconds} seconds...")
                    time.sleep(delay_seconds)
            runtime_error_message = "Unexpected exit from retry loop. This should not happen."
            raise RuntimeError(runtime_error_message)

        return wrapper

    return decorator


@retry(times=5, delay_seconds=5, exception_to_check=BackendError)
def upload_with_retries(handle: str, temp_dir: str, license_name: str) -> None:
    """
    Uploads a model with retries on BackendError indicating the instance slug is already in use.

    Args:
        handle: The model handle.
        temp_dir: Temporary directory where the model is stored.
        license_name: License name for the model.
    """
    model_upload(handle, temp_dir, license_name)


class TestModelUpload(unittest.TestCase):
    def setUp(self) -> None:
        self.temp_dir = tempfile.mkdtemp()
        self.dummy_files = ["dummy_model.h5", "config.json", "metadata.json"]
        for file in self.dummy_files:
            with open(os.path.join(self.temp_dir, file), "w") as f:
                f.write("dummy content")
        self.model_uuid = str(uuid.uuid4())
        credentials = get_kaggle_credentials()
        if not credentials:
            self.fail("Make sure to set your Kaggle credentials before running the tests")
        self.owner_slug = credentials.username
        self.model_slug = f"model_{self.model_uuid}"
        self.handle = f"{self.owner_slug}/{self.model_slug}/pyTorch/new-variation"

    def test_model_upload_and_versioning(self) -> None:
        # Create Instance
        model_upload(self.handle, self.temp_dir, LICENSE_NAME)

        # Create Version
        upload_with_retries(self.handle, self.temp_dir, LICENSE_NAME)

        # If delete model does not raise an error, then the upload was successful.

    def test_model_upload_and_versioning_zip(self) -> None:
        with TemporaryDirectory() as temp_dir:
            for i in range(60):
                test_filepath = Path(temp_dir) / f"temp_test_file_{i}"
                test_filepath.touch()

            # Create Instance
            model_upload(self.handle, temp_dir, LICENSE_NAME)

            # Create Version
            upload_with_retries(self.handle, temp_dir, LICENSE_NAME)

    def test_model_upload_directory(self) -> None:
        with TemporaryDirectory() as temp_dir:
            # Create the new folder within temp_dir
            inner_folder_path = Path(temp_dir) / "inner_folder"
            inner_folder_path.mkdir()

            for i in range(60):
                # Create a file in the temp_dir
                test_filepath = Path(temp_dir) / f"temp_test_file_{i}"
                test_filepath.touch()

                # Create the same file in the inner_folder
                test_filepath_inner = inner_folder_path / f"temp_test_file_{i}"
                test_filepath_inner.touch()

            # Create Instance
            model_upload(self.handle, temp_dir, LICENSE_NAME)

            # Create Version
            upload_with_retries(self.handle, temp_dir, LICENSE_NAME)

    def test_model_upload_nested_dir(self) -> None:
        # Create a nested directory within self.temp_dir
        nested_dir = Path(self.temp_dir) / "nested"
        nested_dir.mkdir()

        # Create dummy files in the nested directory
        nested_dummy_files = ["nested_model.h5", "nested_config.json", "nested_metadata.json"]
        for file in nested_dummy_files:
            with open(nested_dir / file, "w") as f:
                f.write("dummy content in nested file")

        # Call the model upload function with the base directory
        model_upload(self.handle, self.temp_dir, LICENSE_NAME)

    def tearDown(self) -> None:
        models_helpers.delete_model(self.owner_slug, self.model_slug)



import logging
import os
import time
import zipfile
from datetime import datetime
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import List, Union

import requests
from requests.exceptions import ConnectionError, Timeout
from tqdm import tqdm
from tqdm.utils import CallbackIOWrapper

from kagglehub.clients import KaggleApiV1Client
from kagglehub.exceptions import BackendError

logger = logging.getLogger(__name__)

MAX_FILES_TO_UPLOAD = 50
TEMP_ARCHIVE_FILE = "archive.zip"
MAX_RETRIES = 5
REQUEST_TIMEOUT = 600


def parse_datetime_string(string: str) -> Union[datetime, str]:
    time_formats = ["%Y-%m-%dT%H:%M:%S", "%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%dT%H:%M:%S.%f", "%Y-%m-%dT%H:%M:%S.%fZ"]
    for t in time_formats:
        try:
            return datetime.strptime(string[:26], t).replace(microsecond=0)  # noqa: DTZ007
        except:  # noqa: E722, S110
            pass
    return string


class File(object):  # noqa: UP004
    def __init__(self, init_dict: dict) -> None:
        parsed_dict = {k: parse_datetime_string(v) for k, v in init_dict.items()}
        self.__dict__.update(parsed_dict)

    @staticmethod
    def get_size(size: float, precision: int = 0) -> str:
        suffixes = ["B", "KB", "MB", "GB", "TB"]
        suffix_index = 0
        while size >= 1024 and suffix_index < 4:  # noqa: PLR2004
            suffix_index += 1
            size /= 1024.0
        return "%.*f%s" % (precision, size, suffixes[suffix_index])


def _check_uploaded_size(session_uri: str, file_size: int, backoff_factor: int = 1) -> int:
    """Check the status of the resumable upload."""
    headers = {"Content-Length": "0", "Content-Range": f"bytes */{file_size}"}
    retry_count = 0

    while retry_count < MAX_RETRIES:
        try:
            response = requests.put(session_uri, headers=headers, timeout=REQUEST_TIMEOUT)
            if response.status_code == 308:  # Resume Incomplete # noqa: PLR2004
                range_header = response.headers.get("Range")
                if range_header:
                    bytes_uploaded = int(range_header.split("-")[1]) + 1
                    return bytes_uploaded
                return 0  # If no Range header, assume no bytes were uploaded
            else:
                return file_size
        except (ConnectionError, Timeout):
            logger.info(f"Network issue while checking uploaded size, retrying in {backoff_factor} seconds...")
            time.sleep(backoff_factor)
            backoff_factor = min(backoff_factor * 2, 60)
            retry_count += 1

    return 0  # Return 0 if all retries fail


def _upload_blob(file_path: str, model_type: str) -> str:
    """Uploads a file to a remote server as a blob and returns an upload token.

    Parameters
    ==========
    file_path: The path to the file to be uploaded.
    model_type : The type of the model associated with the file.
    """
    file_size = os.path.getsize(file_path)
    data = {
        "type": model_type,
        "name": os.path.basename(file_path),
        "contentLength": file_size,
        "lastModifiedEpochSeconds": int(os.path.getmtime(file_path)),
    }
    api_client = KaggleApiV1Client()
    response = api_client.post("/blobs/upload", data=data)

    # Validate response content
    if "createUrl" not in response:
        create_url_exception = "'createUrl' field missing from response"
        raise BackendError(create_url_exception)
    if "token" not in response:
        token_exception = "'token' field missing from response"
        raise BackendError(token_exception)

    session_uri = response["createUrl"]
    headers = {"Content-Type": "application/octet-stream", "Content-Range": f"bytes 0-{file_size - 1}/{file_size}"}

    retry_count = 0
    uploaded_bytes = 0
    backoff_factor = 1  # Initial backoff duration in seconds

    with open(file_path, "rb") as f, tqdm(total=file_size, desc="Uploading", unit="B", unit_scale=True) as pbar:
        while uploaded_bytes < file_size and retry_count < MAX_RETRIES:
            try:
                f.seek(uploaded_bytes)
                reader_wrapper = CallbackIOWrapper(pbar.update, f, "read")
                headers["Content-Range"] = f"bytes {uploaded_bytes}-{file_size - 1}/{file_size}"
                upload_response = requests.put(
                    session_uri, headers=headers, data=reader_wrapper, timeout=REQUEST_TIMEOUT
                )

                if upload_response.status_code in [200, 201]:
                    return response["token"]
                elif upload_response.status_code == 308:  # Resume Incomplete # noqa: PLR2004
                    uploaded_bytes = _check_uploaded_size(session_uri, file_size)
                else:
                    upload_failed_exception = (
                        f"Upload failed with status code {upload_response.status_code}: {upload_response.text}"
                    )
                    raise BackendError(upload_failed_exception)
            except (requests.ConnectionError, requests.Timeout) as e:
                logger.info(f"Network issue: {e}, retrying in {backoff_factor} seconds...")
                time.sleep(backoff_factor)
                backoff_factor = min(backoff_factor * 2, 60)
                retry_count += 1
                uploaded_bytes = _check_uploaded_size(session_uri, file_size)
                pbar.n = uploaded_bytes  # Update progress bar to reflect actual uploaded bytes

    return response["token"]


def upload_files(source_dir: str, model_type: str) -> List[str]:
    """Zip and Upload directory.
    Parameters
    ==========
    source_dir: the source_dir to upload from
    model_type: Type of the model that is being uploaded.
    """

    with TemporaryDirectory() as temp_dir:
        temp_dir_path = Path(temp_dir)
        zip_path = temp_dir_path / TEMP_ARCHIVE_FILE
        with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zipf:
            source_dir_path = Path(source_dir)
            for file_path in source_dir_path.rglob("*"):
                if file_path.is_file():
                    arcname = file_path.relative_to(source_dir_path)
                    zipf.write(file_path, arcname)

        # Upload the zip file
        return [token for token in [_upload_blob(str(zip_path), model_type)] if token]
